DEFAULT_MAX_RETRIES = 3
DEFAULT_RETRY_MULTIPLIER = 1
DEFAULT_RETRY_MIN_WAIT = 2
DEFAULT_RETRY_MAX_WAIT = 8
DEFAULT_SERVER_URL = "https://api.aci.dev/v1/"
DEFAULT_AFTER_OAUTH2_FLOW_REDIRECT_URL = "https://platform.aci.dev"
